@@ -22,6 +22,60 @@ import traceback
 import psycopg2
 import aioboto3
 import boto3
+from prometheus_client import \
+    CollectorRegistry, Counter, Gauge, Summary, \
+    push_to_gateway, write_to_textfile, start_http_server, exposition
+
+
+@singleton
+class Prometheus:
+    registry = None  # type: CollectorRegistry
+    gauge_instances = {}
+
+    def __init__(self):
+        self.registry = CollectorRegistry()
+
+    def get_registry(self) -> CollectorRegistry:
+        return self.registry
+
+    def start_server(self, **kwargs) -> None:
+        port = 4000 if not 'port' in kwargs else kwargs['port']
+
+        start_http_server(port=port, registry=self.registry)
+
+    def counter(self, name: str, documentation: str) -> Counter:
+        return Counter(name, documentation, registry=self.registry)
+
+    def gauge(self, **kwargs) -> Gauge:
+        r"""
+            :Keyword Arguments:
+                * *new* (``boolean``) --
+                  Return force new object
+                * *name* (``str``) --
+                  Name of Gauge
+                * *documentation* (``str``) --
+                  Documentation of Gauge
+            """
+        if 'new' in kwargs:
+            return Gauge(kwargs['name'], kwargs['documentation'], registry=self.registry)
+
+        if kwargs['name'] not in self.gauge_instances:
+            self.gauge_instances[kwargs['name']] = Gauge(kwargs['name'], kwargs['documentation'],
+                                                         registry=self.registry)
+
+        return self.gauge_instances[kwargs['name']]
+
+    def summary(self, name: str, documentation: str) -> Summary:
+        return Summary(name, documentation, registry=self.registry)
+
+    def push_to_gateway(self, gateway: str, job: str) -> None:
+        push_to_gateway(gateway=gateway, job=job, registry=self.registry)
+
+    def write_to_textfile(self, file: str) -> None:
+        write_to_textfile(file, self.registry)
+
+    def get_output(self) -> str:
+        return exposition.generate_latest(self.get_registry())
 
 
 def log(_func=None, *, logger = None):
@@ -90,7 +144,7 @@ logger = logging.getLogger(__name__)
 
 consul_keys = {"clickhouse": "env/databases/clickhouse", "kafka": "env/databases/kafka",
                "ms-azure-se": "env/databases/ms-azure-se", "loki": "env/databases/loki",
-               "greenplum": "env/databases/greenplum", "storage": "env/databases/storage",}
+               "greenplum": "env/databases/greenplum", "storage": "env/databases/storage"}
 services = {
     "consul": {},
     "aio_consul": {},
@@ -109,6 +163,7 @@ services = {
     "loki_handler": {},
 
     "gp_connection": {},
+    "prometheus": {},
 
     "aiostorage_client": {},
     "aiostorage_resource": {},
@@ -194,9 +249,13 @@ class Factory:
         instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
         return self._consul_client(consulate.Consul, "consul", instance_name, kwargs)
 
-    def aio_consul(self, loop: AbstractEventLoop, **kwargs) -> AioConsul:
+    def aio_consul(self, **kwargs) -> AioConsul:
+        r"""Create and return object of async consul client
+            :Keyword Arguments:
+                * *loop* (``AbstractEventLoop``) --
+                  Custom event loop
+            """
         instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
-        kwargs["loop"] = loop
         return self._consul_client(AioConsul, "aio_consul", instance_name, kwargs)
 
     def _consul_client(self, service_builder, service_name: str, instance_name: str, kwargs):
@@ -372,8 +431,12 @@ class Factory:
         kwargs["target_session_attrs"] = cnfg["target_session_attrs"] if not "target_session_attrs" in kwargs else kwargs["target_session_attrs"]
         return self.get_instance(psycopg2.connect, service_name, instance_name, [], kwargs)
 
-    def storage_client(self, **kwargs):
-        service_name = "storage_client"
+    def prometheus(self, **kwargs) -> Prometheus:
+        service_name = "prometheus"
+        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
+        return self.get_instance(Prometheus, service_name, instance_name)
+
+    def _storage_driver(self, f, service_name, kwargs):
         instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
         scrts = self.secrets["storage"]
         cnfg = self.config["storage"]
@@ -381,39 +444,23 @@ class Factory:
         kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
         kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
         kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
-        return self.get_instance(boto3.Session().client, service_name, instance_name, [], kwargs)
+        return self.get_instance(f, service_name, instance_name, [], kwargs)
+
+
+    def storage_client(self, **kwargs):
+        service_name = "storage_client"
+        return self._storage_driver(boto3.Session().client, service_name, kwargs)
 
 
     def storage_resource(self, **kwargs):
         service_name = "storage_resource"
-        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
-        scrts = self.secrets["storage"]
-        cnfg = self.config["storage"]
-        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
-        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
-        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
-        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
-        return self.get_instance(boto3.resource, service_name, instance_name, [], kwargs)
+        return self._storage_driver(boto3.resource, service_name, kwargs)
 
     def aiostorage_resource(self, **kwargs):
         service_name = "aiostorage_resource"
-        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
-        scrts = self.secrets["storage"]
-        cnfg = self.config["storage"]
-        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
-        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
-        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
-        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
-        return self.get_instance(aioboto3.Session().resource, service_name, instance_name, [], kwargs)
+        return self._storage_driver(aioboto3.Session().resource, service_name, kwargs)
 
     def aiostorage_client(self, **kwargs):
         service_name = "aiostorage_client"
-        instance_name = self.def_instance_name if not self.key_istance_name in kwargs else kwargs[self.key_istance_name]
-        scrts = self.secrets["storage"]
-        cnfg = self.config["storage"]
-        kwargs["service_name"] = "s3" if not "service_name" in kwargs else kwargs["service_name"]
-        kwargs["endpoint_url"] = cnfg["url"] if not "url" in kwargs else kwargs["url"]
-        kwargs["aws_access_key_id"] = scrts["aws_access_key_id"] if not "aws_access_key_id" in kwargs else kwargs["aws_access_key_id"]
-        kwargs["aws_secret_access_key"] = scrts["aws_secret_access_key"] if not "aws_secret_access_key" in kwargs else kwargs["aws_secret_access_key"]
-        return self.get_instance(aioboto3.Session().client, service_name, instance_name, [], kwargs)
+        return self._storage_driver(aioboto3.Session().client, service_name, kwargs)
 
